// Copyright (c)1995 Ray Dream, Inc. All Rights Reserved.
/* $Id: BackFac.cpp 1.1 1996/07/18 23:45:31 Damien Exp $ */

#ifndef __BACKFAC__
#include "BACKFac.h"
#endif

#ifndef __COMBACK__
#include "COMBACK.h"
#endif

#ifndef __BACKDLL__
#include "BackDLL.h"
#endif


// ***** ClassFactory *****
  
SunsetClassFactory::SunsetClassFactory() {
  m_cRef=0L;  // just created so no reference used
  return;
  }


SunsetClassFactory::~SunsetClassFactory() {
  return;
  }

// IUnknown methods of SunsetClassFactory                                                
STDMETHODIMP SunsetClassFactory::QueryInterface(REFIID riid, LPVOID FAR* ppv) {
  *ppv=NULL;

  if (IsEqualIID(riid, IID_IUnknown) || IsEqualIID(riid, IID_IClassFactory))
    *ppv=(LPVOID)this;

  if (*ppv!=NULL) {
    ((LPUNKNOWN)*ppv)->AddRef(); // Add reference if we give a pointer
    return NOERROR;
    }
  else {
    return ResultFromScode(E_NOINTERFACE);
    }
  }


STDMETHODIMP_(ULONG) SunsetClassFactory::AddRef() {
  return ++m_cRef;
  }


STDMETHODIMP_(ULONG) SunsetClassFactory::Release() {
  ULONG   UnreleaseRef;

  UnreleaseRef=--m_cRef;

  if (m_cRef == 0)
    delete this; // No reference left, so delete the SunsetClassFactory

  return UnreleaseRef;
  }

/*
 * SunsetClassFactory::CreateInstance
 *
 * Parameters:
 *  pUnkOuter       LPUNKNOWN to the controlling IUnknown if we are
 *                  being used in an aggregation.
 *  riid            REFIID identifying the interface the caller
 *                  desires to have for the new object.
 *  ppvObj          LPVOID FAR* in which to store the desired
 *                  interface pointer for the new object.
 *
 * Return Value:
 *  HRESULT         NOERROR if successful, otherwise E_NOINTERFACE
 *                  if we cannot support the requested interface.
 */

STDMETHODIMP SunsetClassFactory::CreateInstance(LPUNKNOWN pUnkOuter, REFIID riid, LPVOID FAR* ppvObj) {
  Sunset*      pObj = NULL;
  HRESULT       hr;

  *ppvObj = NULL;
  hr = ResultFromScode(E_OUTOFMEMORY);

  //Verify that a controlling unknown asks for IUnknown
  if (pUnkOuter && !IsEqualIID(riid, IID_IUnknown))
    return ResultFromScode(E_NOINTERFACE);

  //Create the object passing function to notify on destruction.
  if (IsEqualIID(riid, IID_IUnknown) || IsEqualIID(riid, IID_I3DExBackground))
    pObj = new Sunset;
  else
    return ResultFromScode(E_NOINTERFACE);

  hr=pObj->QueryInterface(IID_I3DExBackground, ppvObj);

  //Kill the object if initial creation failed.
  if (FAILED(hr))
    delete pObj;
  else
    global_count_Obj++;

  return hr;
  }


/*
 * SunsetClassFactory::LockServer
 *
 * Purpose:
 *  Increments or decrements the lock count of the DLL.  If the
 *  lock count goes to zero and there are no objects, the DLL
 *  is allowed to unload.  See DllCanUnloadNow in "Sunsetdll.cpp".
 *
 * Parameters:
 *  fLock           BOOL specifying whether to increment or
 *                  decrement the lock count.
 *
 * Return Value:
 *  HRESULT         NOERROR always.
 */

STDMETHODIMP SunsetClassFactory::LockServer(BOOL fLock) {
  if (fLock)
      global_count_Lock++;
  else
      global_count_Lock--;

  return NOERROR;
  }

